(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fortawesome/react-fontawesome'), require('prop-types'), require('react'), require('reactstrap'), require('lemo-object')) :
  typeof define === 'function' && define.amd ? define(['exports', '@fortawesome/react-fontawesome', 'prop-types', 'react', 'reactstrap', 'lemo-object'], factory) :
  (factory((global.LemoForm = {}),global.FontAwesomeIcon,global.PropTypes,global.React,global.Reactstrap,global.LemoObject));
}(this, (function (exports,reactFontawesome,PropTypes,React,reactstrap,lemoObject) { 'use strict';

  PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;
  React = React && React.hasOwnProperty('default') ? React['default'] : React;

  // FontAwesome

  class FieldAddonAppend extends React.Component {
    render() {
      const { icon, text } = this.props;

      if (!icon && !text) {
        return '';
      }

      let content = text;
      if (icon) {
        content = React.createElement(reactFontawesome.FontAwesomeIcon, { icon: icon, fixedWidth: true });
      }

      return React.createElement(
        reactstrap.InputGroupAddon,
        { addonType: 'append' },
        React.createElement(
          reactstrap.InputGroupText,
          null,
          content
        )
      );
    }
  }

  FieldAddonAppend.propTypes = {
    icon: PropTypes.oneOfType([PropTypes.object, PropTypes.array, PropTypes.string]),
    text: PropTypes.string
  };

  // FontAwesome

  class FieldAddonPrepend extends React.Component {
    render() {
      const { icon, text } = this.props;

      if (!icon && !text) {
        return '';
      }

      let content = text;
      if (icon) {
        content = React.createElement(reactFontawesome.FontAwesomeIcon, { icon: icon, fixedWidth: true });
      }

      return React.createElement(
        reactstrap.InputGroupAddon,
        { addonType: 'prepend' },
        React.createElement(
          reactstrap.InputGroupText,
          null,
          content
        )
      );
    }
  }

  FieldAddonPrepend.propTypes = {
    icon: PropTypes.oneOfType([PropTypes.object, PropTypes.array, PropTypes.string]),
    text: PropTypes.string
  };

  // PropTypes

  class FieldFeedback extends React.Component {
    render() {
      const error = this.props.error;

      if (!error) {
        return '';
      }

      return React.createElement(
        reactstrap.FormFeedback,
        null,
        error
      );
    }
  }

  FieldFeedback.propTypes = {
    error: PropTypes.string
  };

  // PropTypes

  const defaultProps = {
    required: false
  };

  const propTypes = {
    name: PropTypes.string,
    required: PropTypes.bool,
    text: PropTypes.string
  };

  class FieldLabel extends React.Component {
    render() {
      const { name, required, text } = this.props;

      if (!text) {
        return '';
      }

      if (!required) {
        return React.createElement(
          reactstrap.Label,
          { 'for': name },
          text
        );
      }

      return React.createElement(
        reactstrap.Label,
        { 'for': name },
        text,
        ' ',
        React.createElement(
          'em',
          { className: 'required' },
          '*'
        )
      );
    }
  }

  FieldLabel.defaultProps = defaultProps;
  FieldLabel.propTypes = propTypes;

  // PropTypes

  class FieldText extends React.Component {
    render() {
      const text = this.props.text;

      if (!text) {
        return '';
      }

      return React.createElement(
        reactstrap.FormText,
        null,
        text
      );
    }
  }

  FieldText.propTypes = {
    text: PropTypes.string
  };

  const FormContext = React.createContext();

  var asyncToGenerator = function (fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  };

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var objectWithoutProperties = function (obj, keys) {
    var target = {};

    for (var i in obj) {
      if (keys.indexOf(i) >= 0) continue;
      if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
      target[i] = obj[i];
    }

    return target;
  };

  // Components

  const withFormApi = Component => React.forwardRef((props, ref) => React.createElement(
    FormContext.Consumer,
    null,
    ({ formApi }) => React.createElement(Component, _extends({ formApi: formApi, ref: ref }, props))
  ));

  // Components

  class Field extends React.Component {
    constructor(props) {
      super(props);

      this.state = {
        disabled: props.disabled,
        error: props.formApi.getError(props.name),
        options: props.options,
        required: props.required,
        validatorsEnabled: props.validatorsEnabled,
        value: props.formApi.getValue(props.name),
        visible: props.visible
      };

      this.handleBlur = this.handleBlur.bind(this);
      this.handleChange = this.handleChange.bind(this);
    }

    componentDidMount() {
      const { formApi } = this.props;

      formApi.set(this.props.name, this);
    }

    componentWillUnmount() {
      const { formApi } = this.props;

      formApi.remove(this.props.name);
    }

    componentWillReceiveProps(nextProps) {
      if (nextProps.disabled !== this.props.disabled) {
        this.setState({
          disabled: nextProps.disabled
        });
      }
      if (nextProps.error !== this.props.error) {
        console.log('Nastavovat pouze pres Form');
        this.setState({
          error: nextProps.error
        });
      }
      if (nextProps.options !== this.props.options) {
        this.setState({
          options: nextProps.options
        });
      }
      if (nextProps.required !== this.props.required) {
        this.setState({
          required: nextProps.required
        });
      }
      if (nextProps.validatorsEnabled !== this.props.validatorsEnabled) {
        this.setState({
          validatorsEnabled: nextProps.validatorsEnabled
        });
      }
      if (nextProps.value !== this.props.value) {
        console.log('Nastavovat pouze pres Form');
        this.setState({
          value: nextProps.value
        });
      }
      if (nextProps.visible !== this.props.visible) {
        this.setState({
          visible: nextProps.visible
        });
      }
    }

    updateState() {
      const { formApi } = this.props;

      this.setState(_extends({}, this.state, {
        error: formApi.getError(this.props.name),
        touched: formApi.getTouched(this.props.name),
        value: formApi.getValue(this.props.name)
      }));
    }

    handleBlur(e) {
      const { formApi } = this.props;

      formApi.setValue(e.target.name, e.target.value);
    }

    handleChange(e) {
      const { formApi } = this.props;

      formApi.setValue(e.target.name, e.target.value);
    }

    render() {
      const {
        addonAppendIcon,
        addonAppendText,
        addonPrependIcon,
        addonPrependText,
        disabledIfEmptyOptions,
        label,
        name,
        optionsEmpty,
        text,
        type
      } = this.props;

      const { error, options, required, visible } = this.state;

      let { disabled, value } = this.state;

      if (!visible) {
        return '';
      }

      let field = null;
      switch (type) {
        case 'select':
          if (true === disabledIfEmptyOptions && 0 === options.length) {
            disabled = true;
          }

          if (0 === options.length) {
            value = '';
          }

          field = React.createElement(
            reactstrap.Input,
            {
              disabled: disabled,
              invalid: !!error,
              name: name,
              onBlur: this.handleBlur,
              onChange: this.handleChange,
              required: required,
              type: type,
              value: value || ''
            },
            optionsEmpty && React.createElement(
              'option',
              { value: '' },
              '-'
            ),
            this.renderOptions()
          );
          break;
        default:
          field = React.createElement(reactstrap.Input, {
            disabled: disabled,
            invalid: !!error,
            name: name,
            onBlur: this.handleBlur,
            onChange: this.handleChange,
            required: required,
            type: type,
            value: !value && value !== 0 ? '' : value
          });
      }

      return React.createElement(
        reactstrap.FormGroup,
        null,
        React.createElement(FieldLabel, { name: name, required: required, text: label }),
        React.createElement(
          reactstrap.InputGroup,
          null,
          React.createElement(FieldAddonPrepend, { icon: addonPrependIcon, text: addonPrependText }),
          field,
          React.createElement(FieldAddonAppend, { icon: addonAppendIcon, text: addonAppendText }),
          React.createElement(FieldFeedback, { error: error })
        ),
        React.createElement(FieldText, { text: text })
      );
    }

    renderOptions() {
      const { options, optionsKey, optionsValue } = this.props;

      return options.map(option => React.createElement(
        'option',
        { key: option[optionsKey], value: option[optionsKey] },
        option[optionsValue]
      ));
    }
  }

  Field.defaultProps = {
    addonAppendIcon: null,
    addonAppendText: null,
    addonPrependIcon: null,
    addonPrependText: null,
    disabled: false,
    disabledIfEmptyOptions: true,
    disabledIfInvalid: [],
    options: [],
    optionsEmpty: true,
    required: false,
    requiredIfNotEmpty: [],
    text: null,
    touched: false,
    type: 'text',
    validateIfChange: [],
    validateOnChange: true,
    validators: [],
    validatorsEnabled: true,
    value: '',
    visible: true
  };

  Field.propTypes = {
    addonAppendIcon: PropTypes.oneOfType([PropTypes.object, PropTypes.array, PropTypes.string]),
    addonAppendText: PropTypes.string,
    addonPrependIcon: PropTypes.oneOfType([PropTypes.object, PropTypes.array, PropTypes.string]),
    addonPrependText: PropTypes.string,
    disabled: PropTypes.bool,
    disabledIfEmptyOptions: PropTypes.bool,
    disabledIfInvalid: PropTypes.array,
    label: PropTypes.string,
    name: PropTypes.string.isRequired,
    onBlur: PropTypes.func,
    onBlurIfValid: PropTypes.func,
    onChange: PropTypes.func,
    onChangeIfValid: PropTypes.func,
    options: PropTypes.oneOfType([PropTypes.array, PropTypes.object]),
    optionsEmpty: PropTypes.bool,
    optionsKey: PropTypes.string,
    optionsValue: PropTypes.string,
    required: PropTypes.bool,
    requiredIfNotEmpty: PropTypes.array,
    ref: PropTypes.func,
    text: PropTypes.string,
    touched: PropTypes.bool,
    type: PropTypes.string,
    validateIfChange: PropTypes.array,
    validateOnChange: PropTypes.bool,
    validators: PropTypes.array,
    validatorsEnabled: PropTypes.bool,
    visible: PropTypes.bool,
    value: PropTypes.oneOfType([PropTypes.number, PropTypes.string])
  };

  var Field$1 = withFormApi(Field);

  // Components

  class Form extends React.Component {
    constructor(props) {
      super(props);

      const { values } = props;

      this.state = {
        isHighlightingErrors: false,
        isSubmitting: false
      };

      this.errors = new lemoObject.DataMap();
      this.fields = new Map();
      this.fieldsDisabledIfInvalid = new Map();
      this.fieldsRequiredIfNotEmpty = new Map();
      this.fieldsTriggerDisabled = new Map();
      this.filled = new lemoObject.DataMap();
      this.touched = new lemoObject.DataMap();
      this.values = new lemoObject.DataMap(values);

      if (props.onInit) {
        props.onInit(this);
      }

      this.get = this.get.bind(this);
      this.getError = this.getError.bind(this);
      this.getErrors = this.getErrors.bind(this);
      this.getFilled = this.getFilled.bind(this);
      this.getTouched = this.getTouched.bind(this);
      this.getValue = this.getValue.bind(this);
      this.getValues = this.getValues.bind(this);
      this.has = this.has.bind(this);
      this.handleChange = this.handleChange.bind(this);
      this.handleSubmit = this.handleSubmit.bind(this);
      this.remove = this.remove.bind(this);
      this.set = this.set.bind(this);
      this.setError = this.setError.bind(this);
      this.setTouched = this.setTouched.bind(this);
      this.setValue = this.setValue.bind(this);
    }

    set(name, field) {
      this.fields.set(name, field);

      // Disabled if not valid
      if (field.props.disabledIfInvalid.length > 0) {
        this.fieldsDisabledIfInvalid.set(name, field.props.disabledIfInvalid);

        field.props.disabledIfInvalid.forEach(fieldToCheck => {
          if (!this.fieldsTriggerDisabled.has(fieldToCheck)) {
            this.fieldsTriggerDisabled.set(fieldToCheck, []);
          }

          this.fieldsTriggerDisabled.get(fieldToCheck).push(name);
        });
      }

      // Required if not empty
      if (field.props.requiredIfNotEmpty.length > 0) {
        this.fieldsRequiredIfNotEmpty.set(name, field.props.requiredIfNotEmpty);
      }

      // Default value
      if (this.values.has(name)) {
        field.setState({
          value: this.values.get(name)
        });
      }
    }

    get(name) {
      return this.fields.get(name);
    }

    has(name) {
      return this.fields.has(name);
    }

    remove(name) {
      this.fields.delete(name);
    }

    setError(name, error) {
      if (null === error) {
        this.errors.delete(name);
      } else {
        this.errors.set(name, error);
      }

      this.fields.get(name).setState({
        error: error
      });
    }

    setTouched(name, value) {
      this.touched.set(name, value);
    }

    setValue(name, value) {
      value = value || null;

      this.values.set(name, value);

      if (this.fields.has(name)) {
        this.handleChange(name, value);
      }
    }

    getValue(name) {
      if (!this.values.has(name)) {
        return null;
      }

      return this.values.get(name);
    }

    getError(name) {
      if (!this.errors.has(name)) {
        return null;
      }

      return this.errors.get(name);
    }

    getErrors() {
      return this.errors;
    }

    getFilled(name = null) {
      if (null !== name) {
        if (this.filled.has(name)) {
          return this.filled.get(name);
        }

        return null;
      }

      return this.filled;
    }

    getTouched(name) {
      if (!this.touched.has(name)) {
        return null;
      }

      return this.touched.get(name);
    }

    getValues() {
      return this.values;
    }

    handleChange(name, value) {
      var _this = this;

      return asyncToGenerator(function* () {
        const field = _this.fields.get(name);

        if (value === field.state.value) {
          return false;
        }

        // Filled
        if ('' === value || null === value) {
          _this.filled.delete(name);
        } else {
          _this.filled.set(name, true);
        }

        // Touched
        _this.touched.set(name, true);

        // Validation
        if (true === field.props.validateOnChange) {
          const error = _this.handleValidate(name, value);

          if (!error) {
            _this.errors.delete(name);
          } else {
            _this.errors.set(name, error);
          }
        }

        // On blur
        if (field.props.hasOwnProperty('onBlur') && typeof field.props.onBlur === 'function') {
          field.props.onBlur(name, value, _this.values.object);
        }

        // On blur if valid
        if (field.props.hasOwnProperty('onBlurIfValid') && typeof field.props.onBlurIfValid === 'function' && !_this.errors.get(name)) {
          field.props.onBlurIfValid(name, value, _this.values.object);
        }

        // On change
        if (field.props.hasOwnProperty('onChange') && typeof field.props.onChange === 'function') {
          field.props.onChange(name, value, _this.values.object);
        }

        // On change if valid
        if (field.props.hasOwnProperty('onChangeIfValid') && typeof field.props.onChangeIfValid === 'function' && !_this.errors.get(name)) {
          field.props.onChangeIfValid(name, value, _this.values.object);
        }

        // Disabled if not valid
        if (_this.fieldsTriggerDisabled.has(name)) {
          yield _this.fieldsTriggerDisabled.get(name).forEach(function (nameToDisable) {
            if (_this.fieldsDisabledIfInvalid.has(nameToDisable)) {
              let isValid = true;
              _this.fieldsDisabledIfInvalid.get(nameToDisable).forEach(function (nameToCheckValue) {
                if (true === isValid && _this.errors.has(nameToCheckValue)) {
                  isValid = false;
                }
              });

              if (true === isValid) {
                _this.fields.get(nameToDisable).setState({
                  disabled: false
                });
              } else {
                _this.errors.delete(nameToDisable);
                _this.values.set(nameToDisable, '');
                _this.fields.get(nameToDisable).setState({
                  error: null,
                  disabled: true,
                  options: [],
                  value: ''
                });
              }
            }
          });
        }

        // Required if not empty
        if (_this.fieldsRequiredIfNotEmpty.has(name)) {
          const namesToCheck = _this.fieldsRequiredIfNotEmpty.get(name);

          // Is some field filled?
          let isRequired = false;
          namesToCheck.forEach(function (nameToCheck) {
            if (false === isRequired) {
              if (!!_this.values.get(nameToCheck)) {
                isRequired = true;
              }
            }
          });

          // Change state of fields
          yield _this.fieldsRequiredIfNotEmpty.forEach(function (field, nameToChange) {
            if (_this.fields.has(nameToChange)) {
              const fieldToChange = _this.fields.get(nameToChange);

              if (fieldToChange.state.required !== isRequired) {
                if (true === isRequired) {
                  fieldToChange.setState({
                    required: isRequired,
                    validatorsEnabled: isRequired
                  });
                } else {
                  fieldToChange.setState({
                    error: null,
                    required: isRequired,
                    validatorsEnabled: isRequired
                  });
                }
              }
            }
          });
        }

        if (_this.props.hasOwnProperty('onChange') && typeof _this.props.onChange === 'function') {
          _this.props.onChange(_this.values.object, name, value);
        }

        field.updateState();
      })();
    }

    handleSubmit(e) {
      var _this2 = this;

      return asyncToGenerator(function* () {
        e.preventDefault(e);

        yield _this2.setState({
          isSubmitting: true
        });

        yield _this2.fields.forEach(function (field, name) {
          let error = _this2.handleValidate(name, _this2.values.get(name));

          _this2.setError(name, error);
        });

        if (_this2.errors.empty()) {
          yield _this2.props.onSubmit(_this2.values.object);
          return;
        }

        yield _this2.setState({
          isHighlightingErrors: true,
          isSubmitting: false
        });

        setTimeout(function () {
          this.setState({
            isHighlightingErrors: false
          });
        }.bind(_this2), 500);
      })();
    }

    handleValidate(name, value = null, validateWithSkip = null) {
      let error = null;
      let field = this.fields.get(name);

      if (false === field.state.validatorsEnabled) {
        return null;
      }
      if (field.state.visible) {
        value = value || '';

        field.props.validators.forEach(validator => {
          if (null === error) {
            error = validator.validate(value.trim(), this.getValues());

            field.props.validateIfChange.forEach(nameToValidate => {
              if (nameToValidate !== validateWithSkip && this.fields.has(nameToValidate)) {
                if (this.touched.has(nameToValidate)) {
                  let fieldError = this.handleValidate(nameToValidate, this.values.get(nameToValidate), name);

                  this.setError(nameToValidate, fieldError);
                }
              }
            });
          }
        });
      }

      return error;
    }

    get formClassName() {
      return this.state.isHighlightingErrors ? 'invalid-highlight' : null;
    }

    get formContext() {
      return {
        formApi: {
          get: this.get,
          getError: this.getError,
          getErrors: this.getErrors,
          getFilled: this.getFilled,
          getTouched: this.getTouched,
          getValue: this.getValue,
          getValues: this.getValues,
          has: this.has,
          isSubmitting: this.state.isSubmitting,
          remove: this.remove,
          set: this.set,
          setError: this.setError,
          setTouched: this.setTouched,
          setValue: this.setValue
        }
      };
    }

    get content() {
      const { children, component, render } = this.props;

      const props = this.formContext;

      if (component) {
        return React.createElement(component, props, children);
      }
      if (render) {
        return render(props);
      }
      if (typeof children === 'function') {
        return children(props);
      }
      return children;
    }

    render() {
      const _props = this.props,
            rest = objectWithoutProperties(_props, ['children', 'component', 'render', 'noValidate', 'onChange', 'onInit', 'onSubmit', 'values']);

      return React.createElement(
        FormContext.Provider,
        { value: this.formContext },
        React.createElement(
          reactstrap.Form,
          _extends({}, rest, {
            className: this.formClassName,
            onSubmit: this.handleSubmit
          }),
          this.content
        )
      );
    }
  }

  Form.defaultProps = {
    noValidate: true,
    onChange: null,
    values: {}
  };

  Form.propTypes = {
    noValidate: PropTypes.bool,
    onInit: PropTypes.func,
    onChange: PropTypes.func,
    onSubmit: PropTypes.func.isRequired,
    values: PropTypes.object
  };

  // HOC

  const buildFieldApi = (formApi, name) => ({
    set: field => formApi.set(name, field),
    remove: () => formApi.remove(name),
    getValue: () => formApi.getValue(name),
    setValue: value => formApi.setValue(name, value),
    getTouched: () => formApi.getTouched(name),
    setTouched: value => formApi.setTouched(name, value),
    getError: () => formApi.getError(name),
    setError: value => formApi.setError(name, value)
  });

  const buildFieldState = (formApi, name) => {
    return {
      disabled: false,
      error: formApi.getError(name),
      touched: formApi.getTouched(name),
      required: false,
      visible: true,
      value: formApi.getValue(name)
    };
  };

  const withField = Component => withFormApi(class extends React.PureComponent {
    constructor(props) {
      super(props);

      const { formApi, name } = props;

      this.fieldApi = buildFieldApi(formApi, name);
      this.fieldState = buildFieldState(formApi, name);

      // Rebuild state when name changes
      this.fieldStateUpdate = () => {
        console.log(this);
        this.setState(buildFieldState(formApi, name));
      };
    }

    render() {
      const rest = objectWithoutProperties(this.props, []);
      return React.createElement(Component, _extends({
        fieldApi: this.fieldApi,
        fieldState: this.fieldState,
        fieldStateUpdate: this.fieldStateUpdate
      }, rest));
    }
  });

  const withFieldApi = field => Component => withFormApi((_ref) => {
    let { formApi } = _ref,
        props = objectWithoutProperties(_ref, ['formApi']);
    return React.createElement(Component, _extends({ fieldApi: buildFieldApi(formApi, field) }, props));
  });

  const withFieldState = field => Component => withFormApi((_ref2) => {
    let { formApi } = _ref2,
        props = objectWithoutProperties(_ref2, ['formApi']);
    return React.createElement(Component, _extends({ fieldState: buildFieldState(formApi, field) }, props));
  });

  exports.Field = Field$1;
  exports.FieldAddonAppend = FieldAddonAppend;
  exports.FieldAddonPrepend = FieldAddonPrepend;
  exports.FieldFeedback = FieldFeedback;
  exports.FieldLabel = FieldLabel;
  exports.FieldText = FieldText;
  exports.Form = Form;
  exports.FormContext = FormContext;
  exports.withField = withField;
  exports.withFieldApi = withFieldApi;
  exports.withFieldState = withFieldState;
  exports.withFormApi = withFormApi;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=lemo-form.umd.js.map
